function sv=gesvd(A),
%  sv=gesvd(A)
% SVD-Zerlegung der Matrix A
% Rueckgabe: Singulaerwerte von A

if nargin==0 A=[1 2 2;2 2 3;3 4 4;4 4 6]; end;
relax=0;

m=size(A,1);n=size(A,2);

maxit=200*n;
sv=zeros(n,1);
% Transformation auf Bidiagonalform mit Householder:

for i=1:n
    a=A(i:end,i);
    if a(1)<0 a(1)=a(1)-norm(a); else a(1)=a(1)+norm(a); end;
    v=[zeros(i-1,1) ; a];
    Q=eye(m)-2*v*v'/(v'*v);
    A=Q*A;
    A(i+1:end,i)=0;    

    if i>=n break; end;
    
    b=A(i,i+1:end)';
    if b(1)<0 b(1)=b(1)-norm(b); else b(1)=b(1)+norm(b); end;
    v=[zeros(i,1) ; b];
    Q=eye(n)-2*v*v'/(v'*v);
    A=A*Q;
    A(i,i+2:end)=0;
end;
B=A(1:n,1:n);

% QR-Iteration auf B mit Givens:
lnd=inf;
for i=1:maxit
    B=givensrot(B',1,1,2)';
    B=givensrot(B,1,1,2);    
    for j=2:n-1
        B=givensrot(B',j-1,j,j+1)';
        B=givensrot(B,j,j,j+1);
    end
    if relax
        % Relaxation: Singulaerwert links oben OK - weiter mit
        % B(2:end,2:ned)
        if abs(B(1,2))<1e-16
            sv(n)=abs(B(1,1));
            B=B(2:n,2:n);
            n=n-1;
            if n==1
                sv(1)=abs(B(1,1));
                break;
            end;
        end;
    else
        % oder einfach Iterieren bis Nebendiagonalelemente nicht mehr
        % kleiner werden
        sv=abs(diag(B));
        nd=max(max(abs(B-diag(diag(B)))));
        if nd==lnd break; end;
        lnd=nd;
    end;
end;
fprintf('Iterationen: %i, Fehler: %e\n',i,norm(sort(sv)-sort(svd(A))));
if (i>=maxit) error('Iteration did not converge'); end;

% Ergebnisse:
% sv=[ 11.5681 1.0174 0.3800 ]';
%
% relax=0: Iterationen: 110, Fehler: 1.122644e-15
% relax=1: Iterationen: 20, Fehler: 1.122644e-15
%